#include <groupchat.h>

void gc_etherx(epacket e)
{
    jpacket p;
    log_debug(ZONE, "[GC] We got a packet from etherx: %s\n", xmlnode2str(e->x));

    switch(e->type)
    {
        case EPACKET_NORMAL:
            xmlnode_hide_attrib(e->x, "etherx:to");
            xmlnode_hide_attrib(e->x, "etherx:from");
            p = jpacket_new(e->x);
            if(p == NULL || jpacket_subtype(p) == JPACKET__ERROR)
            {
                log_debug(ZONE, "invalid or error jpacket");
                xmlnode_free(e->x);
                return;
            }
            gc_breakdance(p);
            break;
        case EPACKET_ERROR:
            log_debug(ZONE, "packet error %s", xmlnode2str(e->x));
            break;
    }
}

void gc_bounce(jpacket p, terror terr)
{
    char *to, *from;
    xmlnode err, x = p->x;
    char code[4];

    log_debug(ZONE,"bouncing packet code %d",terr.code);

    to = xmlnode_get_attrib(x,"to");
    from = xmlnode_get_attrib(x,"from");
    xmlnode_put_attrib(x,"from",to);
    xmlnode_put_attrib(x,"to",from);
    jpacket_reset(p);

    if(p->type == JPACKET_S10N && jpacket_subtype(p) == JPACKET__SUBSCRIBE)
    { /* bounce subscriptions only */
        xmlnode_put_attrib(x,"type","unsubscribed");
        xmlnode_insert_cdata(xmlnode_insert_tag(x,"status"),terr.msg,-1);
        ehandler_send(gc__etherx, x, p->to->server);
        return;
    }

    if(jpacket_subtype(p) == JPACKET__ERROR || p->type == JPACKET_PRESENCE || p->type == JPACKET_S10N)
    { /* presence/error packets are just dropped */
        log_debug(ZONE,"dropping %d packet %s",terr.code,xmlnode2str(x));
        xmlnode_free(x);
        return;
    }

    xmlnode_put_attrib(x,"type","error");
    err = xmlnode_insert_tag(x,"error");

    snprintf(code,4,"%d",terr.code);
    xmlnode_put_attrib(err,"code",code);
    if(terr.msg != NULL)
        xmlnode_insert_cdata(err,terr.msg,strlen(terr.msg));

    ehandler_send(gc__etherx, x, p->to->server);
}

void gc_die()
{
    gcg g;
    gcu u;
    xmlnode x, p;

    x = xmlnode_new_tag("message");
    xmlnode_insert_cdata(xmlnode_insert_tag(x,"body"),"Server Exit",-1);
    p = xmlnode_new_tag("presence");
    xmlnode_put_attrib(p,"type","unavailable");
    for(g = gc__groups->next; g != gc__groups; g = g->next)
    {
        gcg_send(g, NULL, xmlnode_dup(x));
        for(u = g->users->next; u != g->users; u = u->next)
            gcg_send(g, u, xmlnode_dup(p));
    }
    xmlnode_free(p);
    xmlnode_free(x);
}
