#include <groupchat.h>

gcg gc__groups = NULL;

gcg gcg_new(jid id)
{
    char *res;
    gcg g;
    pool p;

    /* clip off any res */
    res = id->resource;
    id->resource = NULL;

    log_debug(ZONE,"Creating new group %s",jid_full(id));

    p = pool_new();
    g = pmalloc(p,sizeof(_gcg));
    g->p = p;
    g->id = jid_new(p,id->server);
    jid_set(g->id,id->user,JID_USER);

    /* setup user ring */
    g->users = pmalloc(p,sizeof(_gcu));
    g->users->next = g->users->prev = g->users;

    /* insert into ring */
    g->next = gc__groups->next;
    gc__groups->next->prev = g;
    g->prev = gc__groups;
    gc__groups->next = g;

    /* restore id */
    id->resource = res;

    return g;
}

gcg gcg_get(jid id)
{
    char *res;
    gcg g;

    log_debug(ZONE,"Fetching group %s",jid_full(id));

    /* clip off any res */
    res = id->resource;
    id->resource = NULL;

    /* find existing group */
    for(g = gc__groups->next; g != gc__groups; g = g->next)
        if(jid_cmp(id,g->id) == 0) break;

    /* restore id */
    id->resource = res;

    if(g == gc__groups)
        return NULL;

    return g;
}

void gcg_send(gcg g, gcu u, xmlnode x)
{
    log_debug(ZONE,"broadcasting to %s from [%s]",jid_full(g->id), xmlnode2str(x));

    if(u == NULL)
        xmlnode_put_attrib(x,"from",jid_full(g->id));
    else
        xmlnode_put_attrib(x,"from",jid_full(u->id));

    for(u = g->users->next; u != g->users; u = u->next)
    {
        xmlnode_put_attrib(x,"to",jid_full(u->user));
        ehandler_send(gc__etherx, xmlnode_dup(x), u->user->server);
    }
    xmlnode_free(x);
}

gcu gcu_get(gcg g, jid id)
{
    gcu u;

    if(g == NULL) return NULL;

    log_debug(ZONE,"User serch %s in %s",jid_full(id),jid_full(g->id));

    for(u = g->users->next; u != g->users; u = u->next)
        if(jid_cmp(id,u->user) == 0) return u;

    return NULL;
}

gcu gcu_nick(gcg g, char *nick)
{
    gcu u;

    if(g==NULL||nick==NULL||nick[0]=='\0') return NULL;

    log_debug(ZONE,"Nick serch %s in %s",nick,jid_full(g->id));

    for(u = g->users->next; u != g->users; u = u->next)
        if(strcasecmp(u->nick,nick) == 0) return u;

    return NULL;
}

gcu gcg_user(gcg g, jid id, char *nick)
{
    gcu u;
    xmlnode x, body;
    pool p;

    if(g == NULL) return NULL;

    log_debug(ZONE,"Checking user %s in group %s with nick %s",jid_full(id),jid_full(g->id),nick);

    u = gcu_get(g, id);

    /* change nickname */
    if(u != NULL)
    {
        /* send unavail for old nick */
        x = xmlnode_new_tag("presence");
        xmlnode_put_attrib(x,"type","unavailable");
        gcg_send(g, u, x);

        /* msg notice from the server */
        x = xmlnode_new_tag("message");
        xmlnode_put_attrib(x,"type","groupchat");
        body = xmlnode_insert_tag(x,"body");
        xmlnode_insert_cdata(body,u->nick,-1);
        xmlnode_insert_cdata(body," is now known as ",-1);
        xmlnode_insert_cdata(body,nick,-1);
        gcg_send(g, NULL, x);

        /* update settings */
        u->nick = pstrdup(u->p,nick);
        jid_set(u->id,nick,JID_RESOURCE);
        return u;
    }

    p = pool_new();
    u = pmalloc(p,sizeof(_gcu));
    u->p = p;
    u->pres = NULL;
    u->nick = pstrdup(p,nick);
    u->user = jid_new(p,jid_full(id));
    u->id = jid_new(p,jid_full(g->id));
    jid_set(u->id,nick,JID_RESOURCE);

    /* insert into ring */
    u->next = g->users->next;
    g->users->next->prev = u;
    u->prev = g->users;
    g->users->next = u;

    /* msg notice from the server */
    x = xmlnode_new_tag("message");
    xmlnode_put_attrib(x,"type","groupchat");
    body = xmlnode_insert_tag(x,"body");
    xmlnode_insert_cdata(body,u->nick,-1);
    xmlnode_insert_cdata(body," has arrived",-1);
    gcg_send(g, NULL, x);

    return u;
}

void gcu_del(gcg g, gcu u)
{
    xmlnode x, body;

    /* unlink the user */
    u->prev->next = u->next;
    u->next->prev = u->prev;
    xmlnode_free(u->pres);

    /* msg notice from the server */
    x = xmlnode_new_tag("message");
    xmlnode_put_attrib(x,"type","groupchat");
    body = xmlnode_insert_tag(x,"body");
    xmlnode_insert_cdata(body,u->nick,-1);
    xmlnode_insert_cdata(body," has left",-1);
    gcg_send(g, NULL, x);

    pool_free(u->p);
}

